/*******************************************************************
  Key Value Persistent Storage

  keyval_dbuff.c	- dynamic buffer storage/retrieval

  (C) Copyright 2003-2005 by Pavel Pisa - Originator

  The uLan utilities library can be used, copied and modified under
  next licenses
    - GPL - GNU General Public License
    - LGPL - GNU Lesser General Public License
    - MPL - Mozilla Public License
    - and other licenses added by project originators
  Code can be modified and re-distributed under any combination
  of the above listed licenses. If contributor does not agree with
  some of the licenses, he/she can delete appropriate line.
  Warning, if you delete all lines, you are not allowed to
  distribute source code and/or binaries utilizing code.
  
  See files COPYING and README for details.

 *******************************************************************/

#include <inttypes.h>
#include <string.h>
#include <ul_dbuff.h>
#include <keyvalpb.h>
#include <keyval_dbuff.h>

/**
 * kvpb_get_key - Retrieve value for given key ID into dynamic data buffer
 * @store: Pointer to the KVPB access information/state structure
 * @keyid: Ordinal value representing key ID
 * @dbuf: Pointer to the dynamic data buffer
 *
 * Return Value: Number of retrieved value bytes if operation is successful
 *	or -1 if there is no such key ID or operation fails for other reason.
 *	If space for buffer cannot be allocated -3 is returned.
 * File: keyval_dbuff.c
 */
int kvpb_get_key_dbuff(kvpb_block_t *store, kvpb_keyid_t keyid, ul_dbuff_t *dbuf)
{
  kvpb_size_t size=0;
  int  res=0;

  do{
    if(size){
      if(ul_dbuff_prep(dbuf, size)!=size)
	return -3;
    }

    res=kvpb_get_key(store, keyid, dbuf->capacity, dbuf->data);
    if(res<0){
      ul_dbuff_prep(dbuf, 0);
      return res;
    }
    if(dbuf->capacity>=res){
      dbuf->len=res;
      break;
    }
    size=res;
  }while(1);

  return res;
}

/**
 * kvpb_set_key_dbuff - Store value from given dynamic data buffer to the specified key ID
 * @store: Pointer to the KVPB access information/state structure
 * @keyid: Ordinal value representing key ID
 * @dbuf: Pointer to the dynamic data buffer
 *
 * Return Value: Number of stored value bytes if operation is successful
 *	or -1 if value cannot be stored.
 * File: keyval_dbuff.c
 */
int kvpb_set_key_dbuff(kvpb_block_t *store, kvpb_keyid_t keyid, ul_dbuff_t *dbuf)
{
  return kvpb_set_key(store, keyid, dbuf->len, dbuf->data);
}


